@@ -2,17 +2,26 @@ import os
 import re
 from contextlib import contextmanager
 from itertools import cycle
-
-try:
-    from invoke.vendor.six import raise_from, iteritems, string_types
-except ImportError:
-    from six import raise_from, iteritems, string_types
+from os import PathLike
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Generator,
+    Iterator,
+    List,
+    Optional,
+    Union,
+)
+from unittest.mock import Mock
 
 from .config import Config, DataProxy
 from .exceptions import Failure, AuthFailure, ResponseNotAccepted
 from .runners import Result
 from .watchers import FailingResponder
 
+if TYPE_CHECKING:
+    from invoke.runners import Runner
+
 
 class Context(DataProxy):
     """
@@ -34,7 +43,7 @@ class Context(DataProxy):
     .. versionadded:: 1.0
     """
 
-    def __init__(self, config=None):
+    def __init__(self, config: Optional[Config] = None) -> None:
         """
         :param config:
             `.Config` object to use as the base configuration.
@@ -55,22 +64,22 @@ class Context(DataProxy):
         #: A list of commands to run (via "&&") before the main argument to any
         #: `run` or `sudo` calls. Note that the primary API for manipulating
         #: this list is `prefix`; see its docs for details.
-        command_prefixes = list()
+        command_prefixes: List[str] = list()
         self._set(command_prefixes=command_prefixes)
         #: A list of directories to 'cd' into before running commands with
         #: `run` or `sudo`; intended for management via `cd`, please see its
         #: docs for details.
-        command_cwds = list()
+        command_cwds: List[str] = list()
         self._set(command_cwds=command_cwds)
 
     @property
-    def config(self):
+    def config(self) -> Config:
         # Allows Context to expose a .config attribute even though DataProxy
         # otherwise considers it a config key.
         return self._config
 
     @config.setter
-    def config(self, value):
+    def config(self, value: Config) -> None:
         # NOTE: mostly used by client libraries needing to tweak a Context's
         # config at execution time; i.e. a Context subclass that bears its own
         # unique data may want to be stood up when parameterizing/expanding a
@@ -78,7 +87,7 @@ class Context(DataProxy):
         # runtime.
         self._set(_config=value)
 
-    def run(self, command, **kwargs):
+    def run(self, command: str, **kwargs: Any) -> Optional[Result]:
         """
         Execute a local shell command, honoring config options.
 
@@ -97,11 +106,13 @@ class Context(DataProxy):
     # NOTE: broken out of run() to allow for runner class injection in
     # Fabric/etc, which needs to juggle multiple runner class types (local and
     # remote).
-    def _run(self, runner, command, **kwargs):
+    def _run(
+        self, runner: "Runner", command: str, **kwargs: Any
+    ) -> Optional[Result]:
         command = self._prefix_commands(command)
         return runner.run(command, **kwargs)
 
-    def sudo(self, command, **kwargs):
+    def sudo(self, command: str, **kwargs: Any) -> Optional[Result]:
         """
         Execute a shell command via ``sudo`` with password auto-response.
 
@@ -174,7 +185,9 @@ class Context(DataProxy):
         return self._sudo(runner, command, **kwargs)
 
     # NOTE: this is for runner injection; see NOTE above _run().
-    def _sudo(self, runner, command, **kwargs):
+    def _sudo(
+        self, runner: "Runner", command: str, **kwargs: Any
+    ) -> Optional[Result]:
         prompt = self.config.sudo.prompt
         password = kwargs.pop("password", self.config.sudo.password)
         user = kwargs.pop("user", self.config.sudo.user)
@@ -227,10 +240,8 @@ class Context(DataProxy):
             # For now that has been judged unnecessary complexity.
             if isinstance(failure.reason, ResponseNotAccepted):
                 # NOTE: not bothering with 'reason' here, it's pointless.
-                # NOTE: using raise_from(..., None) to suppress Python 3's
-                # "helpful" multi-exception output. It's confusing here.
                 error = AuthFailure(result=failure.result, prompt=prompt)
-                raise_from(error, None)
+                raise error
             # Reraise for any other error so it bubbles up normally.
             else:
                 raise
@@ -238,7 +249,7 @@ class Context(DataProxy):
     # TODO: wonder if it makes sense to move this part of things inside Runner,
     # which would grow a `prefixes` and `cwd` init kwargs or similar. The less
     # that's stuffed into Context, probably the better.
-    def _prefix_commands(self, command):
+    def _prefix_commands(self, command: str) -> str:
         """
         Prefixes ``command`` with all prefixes found in ``command_prefixes``.
 
@@ -253,7 +264,7 @@ class Context(DataProxy):
         return " && ".join(prefixes + [command])
 
     @contextmanager
-    def prefix(self, command):
+    def prefix(self, command: str) -> Generator[None, None, None]:
         """
         Prefix all nested `run`/`sudo` commands with given command plus ``&&``.
 
@@ -309,7 +320,7 @@ class Context(DataProxy):
             self.command_prefixes.pop()
 
     @property
-    def cwd(self):
+    def cwd(self) -> str:
         """
         Return the current working directory, accounting for uses of `cd`.
 
@@ -329,10 +340,10 @@ class Context(DataProxy):
         # TODO: see if there's a stronger "escape this path" function somewhere
         # we can reuse. e.g., escaping tildes or slashes in filenames.
         paths = [path.replace(" ", r"\ ") for path in self.command_cwds[i:]]
-        return os.path.join(*paths)
+        return str(os.path.join(*paths))
 
     @contextmanager
-    def cd(self, path):
+    def cd(self, path: Union[PathLike, str]) -> Generator[None, None, None]:
         """
         Context manager that keeps directory state when executing commands.
 
@@ -393,11 +404,9 @@ class MockContext(Context):
     Primarily useful for testing Invoke-using codebases.
 
     .. note::
-        If this class' constructor is able to import the ``Mock`` class at
-        runtime (via the ``mock`` or ``unittest.mock`` modules, in that order)
-        it will wraps its ``run``, etc methods in ``Mock`` objects. This allows
-        you to easily assert that the methods (still returning the values you
-        prepare them with) were actually called.
+        This class wraps its ``run``, etc methods in `unittest.mock.Mock`
+        objects. This allows you to easily assert that the methods (still
+        returning the values you prepare them with) were actually called.
 
     .. note::
         Methods not given `Results <.Result>` to yield will raise
@@ -406,10 +415,10 @@ class MockContext(Context):
 
     .. versionadded:: 1.0
     .. versionchanged:: 1.5
-        Added conditional ``Mock`` wrapping of ``run`` and ``sudo``.
+        Added ``Mock`` wrapping of ``run`` and ``sudo``.
     """
 
-    def __init__(self, config=None, **kwargs):
+    def __init__(self, config: Optional[Config] = None, **kwargs: Any) -> None:
         """
         Create a ``Context``-like object whose methods yield `.Result` objects.
 
@@ -449,7 +458,7 @@ class MockContext(Context):
             Similarly, iterable results are normally exhausted once, but when
             this setting is enabled, they are wrapped in `itertools.cycle`.
 
-            Default: ``False`` (for backwards compatibility reasons).
+            Default: ``True``.
 
         :raises:
             ``TypeError``, if the values given to ``run`` or other kwargs
@@ -461,29 +470,20 @@ class MockContext(Context):
             Added support for regex dict keys.
         .. versionchanged:: 1.5
             Added the ``repeat`` keyword argument.
+        .. versionchanged:: 2.0
+            Changed ``repeat`` default value from ``False`` to ``True``.
         """
-        # Figure out if we can support Mock in the current environment
-        Mock = None
-        try:
-            from mock import Mock
-        except ImportError:
-            try:
-                from unittest.mock import Mock
-            except ImportError:
-                pass
         # Set up like any other Context would, with the config
-        super(MockContext, self).__init__(config)
+        super().__init__(config)
         # Pull out behavioral kwargs
-        # TODO 2.0: Jesus tap-dancing Christ this needs to default to True, it
-        # gets me every single time
-        self._set("__repeat", kwargs.pop("repeat", False))
+        self._set("__repeat", kwargs.pop("repeat", True))
         # The rest must be things like run/sudo - mock Context method info
-        for method, results in iteritems(kwargs):
+        for method, results in kwargs.items():
             # For each possible value type, normalize to iterable of Result
             # objects (possibly repeating).
-            singletons = tuple([Result, bool] + list(string_types))
+            singletons = (Result, bool, str)
             if isinstance(results, dict):
-                for key, value in iteritems(results):
+                for key, value in results.items():
                     results[key] = self._normalize(value)
             elif isinstance(results, singletons) or hasattr(
                 results, "__iter__"
@@ -495,20 +495,19 @@ class MockContext(Context):
                 raise TypeError(err.format(type(results)))
             # Save results for use by the method
             self._set("__{}".format(method), results)
-            # Wrap the method in a Mock, if applicable
-            if Mock is not None:
-                self._set(method, Mock(wraps=getattr(self, method)))
+            # Wrap the method in a Mock
+            self._set(method, Mock(wraps=getattr(self, method)))
 
-    def _normalize(self, value):
+    def _normalize(self, value: Any) -> Iterator[Any]:
         # First turn everything into an iterable
-        if not hasattr(value, "__iter__") or isinstance(value, string_types):
+        if not hasattr(value, "__iter__") or isinstance(value, str):
             value = [value]
         # Then turn everything within into a Result
         results = []
         for obj in value:
             if isinstance(obj, bool):
                 obj = Result(exited=0 if obj else 1)
-            elif isinstance(obj, string_types):
+            elif isinstance(obj, str):
                 obj = Result(obj)
             results.append(obj)
         # Finally, turn that iterable into an iteratOR, depending on repeat
@@ -519,7 +518,7 @@ class MockContext(Context):
     # worth. Maybe in situations where Context grows a _lot_ of methods (e.g.
     # in Fabric 2; though Fabric could do its own sub-subclass in that case...)
 
-    def _yield_result(self, attname, command):
+    def _yield_result(self, attname: str, command: str) -> Result:
         try:
             obj = getattr(self, attname)
             # Dicts need to try direct lookup or regex matching
@@ -529,7 +528,7 @@ class MockContext(Context):
                 except KeyError:
                     # TODO: could optimize by skipping this if not any regex
                     # objects in keys()?
-                    for key, value in iteritems(obj):
+                    for key, value in obj.items():
                         if hasattr(key, "match") and key.match(command):
                             obj = value
                             break
@@ -539,30 +538,33 @@ class MockContext(Context):
             # Here, the value was either never a dict or has been extracted
             # from one, so we can assume it's an iterable of Result objects due
             # to work done by __init__.
-            result = next(obj)
+            result: Result = next(obj)
             # Populate Result's command string with what matched unless
             # explicitly given
             if not result.command:
                 result.command = command
             return result
         except (AttributeError, IndexError, KeyError, StopIteration):
-            raise_from(NotImplementedError(command), None)
+            # raise_from(NotImplementedError(command), None)
+            raise NotImplementedError(command)
 
-    def run(self, command, *args, **kwargs):
+    def run(self, command: str, *args: Any, **kwargs: Any) -> Result:
         # TODO: perform more convenience stuff associating args/kwargs with the
         # result? E.g. filling in .command, etc? Possibly useful for debugging
         # if one hits unexpected-order problems with what they passed in to
         # __init__.
         return self._yield_result("__run", command)
 
-    def sudo(self, command, *args, **kwargs):
+    def sudo(self, command: str, *args: Any, **kwargs: Any) -> Result:
         # TODO: this completely nukes the top-level behavior of sudo(), which
         # could be good or bad, depending. Most of the time I think it's good.
         # No need to supply dummy password config, etc.
         # TODO: see the TODO from run() re: injecting arg/kwarg values
         return self._yield_result("__sudo", command)
 
-    def set_result_for(self, attname, command, result):
+    def set_result_for(
+        self, attname: str, command: str, result: Result
+    ) -> None:
         """
         Modify the stored mock results for given ``attname`` (e.g. ``run``).
 
